import com.mxgxw.graphics.*;
import com.mxgxw.games.*;
import java.awt.event.*;
import java.awt.*;
import java.applet.*;
import javax.swing.*;


/**
 * Applet que muestra la interfaz y maneja los eventos para el juego BuscaMinas
 * @author Mario Gmez
 * @version 0.1.0
 * @license Cdigo fuente protegido bajo la GNU-GPL o posterior que puede ser obtenida de http://www.fsf.org/. Se incluye una copia con las fuentes en el archivo LICENSE.
 */
public class BuscaMinasApplet extends Applet implements ActionListener {
    
    /**
     * Constante que define cuando el juego se encuentra en una partida.
     */
    public static final int ESTADO_JUEGO = 0;
    /**
     * Constante que define cuando el juego ha finalizado y la partida
     * se ha ganado.
     */
    public static final int ESTADO_ESPERA = 1;
    /**
     * Constante que define cuando el juego ha finalizado y la partida
     * se ha perdido.
     */
    public static final int ESTADO_ESPERA2 = 2;
    
    // Matriz de los botones, se utiliza para generar la interfaz grfica
    // y para manejar de una manera sencilla el comportamiento de la misma
    private BotonPosicion matriz[][];
    
    // Imagen de una bomba
    private Image bomb;
    
    // Imagen de una explosin
    private Image expl;
    
    // Clase que maneja el juego Buscaminas
    private BuscaMinas bm;
    
    // Define el estado actual del juego
    private int estadoJuego;
    
    // Cuenta el numero de clicks realizados
    private int clicks;
    
    // Botn de la interface
    private JButton comando;
    // Caja de texto de la interface
    private JTextField estado;
    
    // Clase que administra los puntajes
    private AdministradorPuntajes ap;
    
    /**
     * Inicializa todas las variables del applet y define la forma y comportamiento
     * de la interfaz grfica.
     */
    public void init() {
        
        // Las siguientes tres lineas activan el tema por defecto para swing
        // en este caso se activa el tema de windows XP si esta habilitado
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }catch(Exception e) {}
        
        // Paneles de colocacin de los componentes
        // En horiz colocaremos los componentes que querramos que esten
        // en posicin horizontal, juego es un panel al colocaremos
        // todos sus elementos en posicin vertical
        Panel horiz,juego;
        
        // Iteradores para generar la matriz de botones
        int i,j;
        
        // Creacin de la matriz de botones
        matriz = new BotonPosicion[10][10];
        
        // Creacin de los elementos de la interfaz
        estado = new JTextField(10); // Caja de texto
        estado.setEditable(false); // Edicin deshabilitada por defecto
        comando = new JButton("Nuevo Juego"); // Boton de juego
        comando.addActionListener(this); // Manejador de evento
        
        // Los gridbag son los que posicionan los elementos
        // de forma horizontal o vertical, tenemos dos para facilitar
        // el uso
        GridBagLayout gridbag1 = new GridBagLayout();
        GridBagLayout gridbag2 = new GridBagLayout();
        // GridBagConstraints define la forma en que se ubicara un objeto
        // especfico.
        GridBagConstraints c = new GridBagConstraints();
        
        // Creacin de los nuevos paneles utilizando la plantilla
        // de posicionamiento del gridbag
        juego = new Panel(gridbag1);
        horiz = new Panel(gridbag2);
        
        this.setBackground(Color.white);
        horiz.setBackground(Color.white);
        juego.setBackground(Color.white);
        // Definimos que el proximo objeto tendra una posicin horizontal
        c.fill = GridBagConstraints.HORIZONTAL;
        // Definimos las caracteristicas de posicin para la caja estado
        gridbag2.setConstraints(estado,c);
        // Agregamos la caja estado al panel horizontal
        horiz.add(estado);
        // Establecemos las mismas caracteristicas para el boton comando,
        // esto hara que el componente se agregue horizontalmente a la par
        // del anterior
        gridbag2.setConstraints(comando,c);
        // Agregamos el boton comando
        horiz.add(comando);
        
        // Definimos nuevas caracteristicas de posicin para el siguiente objeto
        // El objeto sera posicionado verticalmente
        c.fill = GridBagConstraints.VERTICAL;
        // El objeto sera el ultimo de la fila esto hara que los demas objetos
        // sean colocados debajo de este en forma vertical.
        c.gridwidth = GridBagConstraints.REMAINDER;
        // Establecemos las caracteristicas de posicin del panel horiz
        gridbag1.setConstraints(horiz,c);
        // Agregamos el panel horiz al panel juego
        juego.add(horiz);
        
        /**
         * El siguiente ciclo crea los botones y les asigna su posicin dentro
         * del arreglo, cada objeto creado se aade a un nuevo panel horizontal
         * y a su vez cada panel horizontal es agregado al panel vertical.
         */
        for(i=0;i<10;i++) {
            horiz = new Panel(gridbag2); // Nuevo panel horizontal
            for(j=0;j<10;j++) {
                matriz[i][j] = new BotonPosicion(new Point(j,i)); // Crea nuevo boton
                matriz[i][j].addActionListener(this); // Agrega el manejador de accin
                horiz.add(matriz[i][j]); // Agrega el boton al panel
            }
            c.fill = GridBagConstraints.VERTICAL; // Define la posicion
            c.gridwidth = GridBagConstraints.REMAINDER; //Fin de fila
            gridbag1.setConstraints(horiz,c); // Define la posicion
            juego.add(horiz); // Agrega la fila al panel vertical
        }
        
        // Aade al applet el panel de juego
        this.add(juego);
        
        // Cargamos las imagenes a utilizar en el juego.
        bomb = this.getImage(this.getDocumentBase(),"bomb.png");
        expl = this.getImage(this.getDocumentBase(),"bomba.png");
        
        // Generamos un nuevo juego de buscaminas
        bm = new BuscaMinas();
        // Minamos el campo
        bm.llenarCampo();
        
        // Definimos el estado a partida nueva
        estadoJuego = BuscaMinasApplet.ESTADO_JUEGO;
        // Ponemos el contador de clicks a cero
        clicks = 0;
        
        // Generamos un nuevo administrador de puntajes
        ap = new AdministradorPuntajes();
    }
    
    /**
     * Dibuja el componente
     * @param g Entorno grfico en que se dibujaran los componentes
     */
    public void paint(Graphics g) {
        g.setColor(Color.white);
        g.fillRect(0,0,this.getWidth(), this.getHeight());
    }
    
    /**
     * Reinicia el juego a sus valores por defecto
     */
    private void reset() {
        // Reinicia el juego
        bm.reset();
        int i,j;
        
        // Pone los valores por defecto de los botones:
        for(i=0;i<10;i++){
            for(j=0;j<10;j++){
                matriz[j][i].setText("");
                matriz[j][i].cambiarTipoFondo(BotonPosicion.VALOR_COLOR);
                matriz[j][i].ponerColorBrilloFondo(new Color(0xE1,0xE1,0xE1));
                matriz[j][i].ponerEstado(BotonPosicion.ELEVADO);
                matriz[j][i].ponerColorTexto(new java.awt.Color(0xCC,0x00,0x00));
            }
        }
        // Contador de clicks a cero
        clicks = 0;
        
        // Deshabilita el campo de texto
        estado.setText("");
        estado.setEditable(false);
        // Habilita el boton de comando
        comando.setText("Nuevo Juego");
        
        // Minamos el campo
        bm.llenarCampo();
        
        // Ponemos el juego en una nueva partida
        estadoJuego = BuscaMinasApplet.ESTADO_JUEGO;
    }
    
    /**
     * Funcin recursiva que se encarga de descubrir a los cuadros adyacentes que
     * no tienen bomba, cuando esta funcin se llama por primera vez es necesario
     * establecer "primero" a true para indicar que es de donde se esta generando
     * el evento.
     * La funcin busca recursivamente si la casilla que esta revisando tiene bombas
     * si no tiene bombas cerca busca en las casillas aledaas.
     * Si tiene bombas en casillas aledaas, coloca el numero correspondiente a la
     * funcin radar del juego BuscaMinas.
     */
    private void buscarCasillas(Point pos,boolean primero) {
        int i,j,x,y,radar;
        
        // Revisamos que el rango sea correcto:
        if(bm.revisarRango(pos.x,10)&&bm.revisarRango(pos.y,10)) {
            // La funcin se comparar cuando el botn que se evalua no se haya presionado
            // asi evitamos que se revise el mismo boton dos veces. Tambien se revisar cuando
            // definamos que el boton que se presiona es el que esta llamando a la funcin.
            if(matriz[pos.y][pos.x].obtenerEstado()==BotonPosicion.ELEVADO||primero) {
                // Este ciclo generar las posiciones para los nueve cuadros incluyendo
                // al que se esta verificando.
                for(i=-1;i<2;i++){
                    for(j=-1;j<2;j++){
                        // Calculamos las posiciones a revisar en base a la posicin actual
                        x = pos.x+j;
                        y = pos.y+i;
                        // Antes de evaluar revisamos si la nueva posicin es vlida
                        if(bm.revisarRango(x,10)&&bm.revisarRango(y,10)&&!(j==0&&i==0)) {
                            // Si no hay mina en la casilla que revisamos
                            if(!bm.hayMina(pos)) {
                                // Obtenemos la cantidad de minas
                                radar = bm.radar(pos);
                                // Si la cantidad de minas aledaas es cero:
                                if(radar==0) {
                                    // Ponemos el texto del boton vacio
                                    matriz[pos.y][pos.x].setText("");
                                    // Presionamos el boton
                                    matriz[pos.y][pos.x].ponerEstado(BotonPosicion.PRESIONADO);
                                    // Llamada recursiva a la funcin buscarCasillas
                                    buscarCasillas(new Point(x,y),false);
                                } else { // Si hay una o mas minas aledaas
                                    // Dependiendo de la cantidad de minas establecemos
                                    // el color del botn:
                                    switch(radar) {
                                        case 1: case 2:
                                            matriz[pos.y][pos.x].ponerColorTexto(Color.blue);
                                            break;
                                        case 3: case 4: case 5: case 6: case 7: case 8:
                                            matriz[pos.y][pos.x].ponerColorTexto(Color.red);
                                            break;
                                    }
                                    // Establecemos el texto del botn al numero de minas
                                    matriz[pos.y][pos.x].setText(String.valueOf(radar));
                                    // Establecemos que se mostrar el texto como fondo
                                    matriz[pos.y][pos.x].cambiarTipoFondo(BotonPosicion.VALOR_TEXTO);
                                    // Presionamos el botn
                                    matriz[pos.y][pos.x].ponerEstado(BotonPosicion.PRESIONADO);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Busca en el arreglo los botones que tengan minas, pone la imagen de bomba
     * y muestra el dibujo. Esta funcion se muestra siempre al terminar un juego.
     */
    private void buscarMinas() {
        int i,j;
        // Revisamos todas las posiciones del arreglo:
        for(i=0;i<10;i++){
            for(j=0;j<10;j++){
                // Si el botn esta elevado
                if(matriz[j][i].obtenerEstado()==BotonPosicion.ELEVADO) {
                    // Si hay una mina en esa posicin
                    if(bm.hayMina(new Point(i,j))) {
                        // Colocar la imagen de la bomba
                        matriz[j][i].ponerImagen(bomb);
                        // Cambia el tipo de fondo del boton
                        matriz[j][i].cambiarTipoFondo(BotonPosicion.VALOR_IMAGEN);
                    }
                }
            }
        }
        
    }
    
    /**
     * Revisa si ya se encontraron todas las bombas,si es asi se termina el juego
     */
    private void revisarBombas() {
        int i,j,contador = 0;
        // Recorre la matriz buscando los botones que estan elevados
        for(i=0;i<10;i++){
            for(j=0;j<10;j++){
                if(matriz[j][i].obtenerEstado()==BotonPosicion.ELEVADO) {
                    contador++;
                }
            }
        }
        // Si el numero de botones es 10 significa que se encontraron todas
        // las minas.
        if(contador==10) {
            // Ponemos el juego en espera
            estadoJuego = BuscaMinasApplet.ESTADO_ESPERA;
            // Habilitamos la caja de texto
            estado.setText("");
            estado.setEditable(true);
            // Cambiamos el texto del boton de comando
            comando.setText("Guardar");
            // Mostramos las minas
            buscarMinas();
        }
    }
    
    /**
     * Funcion que maneja los eventos de click de ratn en los componentes.
     * @param e Evento generado por el Componente
     */
    public void actionPerformed(ActionEvent e) {
        // Obtenemos el objeto que gener el evento
        Object obj = e.getSource();
        // Lo utilizaremos luego al trabajar con los botones
        BotonPosicion bt;
        
        // Revisamos si el objeto que gener el evento
        // es una instancia de BotonPosicin, es decir
        // revisamos si es un boton de la matriz
        if(obj instanceof BotonPosicion) {
            // El comportamiento de estos botones solo estar activo
            // cuando el estado del juego este en medio de una partida
            switch(estadoJuego) {
                case BuscaMinasApplet.ESTADO_JUEGO:
                    // Realizamos el casting para facilitar la escritura
                    bt = ((BotonPosicion)obj);
                    
                    // Si el boton no esta presionado
                    if(bt.obtenerEstado()!=BotonPosicion.PRESIONADO) {
                        // Cambiamos el botn a presionado
                        bt.ponerEstado(BotonPosicion.PRESIONADO);
                        // Si hay una mina:
                        if(bm.hayMina(bt.obtenerPosicion())) {
                            // Cambiamos el color del fondo del botn
                            bt.ponerColorBrilloFondo(Color.gray);
                            // Ponemos la imagen de explosin
                            bt.ponerImagen(expl);
                            // Mostramos la imagen en el botn
                            bt.cambiarTipoFondo(BotonPosicion.VALOR_IMAGEN);
                            // Mostramos las minas restantes
                            buscarMinas();
                            // Habilitamos el botn comando con el texto Nuevo Juego
                            comando.setText("Nuevo Juego");
                            // Cambiamos el estado de juego a ESPERA2
                            estadoJuego = BuscaMinasApplet.ESTADO_ESPERA2;
                        } else {
                            // Si no hay una bomba, llamamos a la funcion
                            // buscarCasillas mandandole la posicin actual del
                            // boton y especificandole que ser esta funcin ser
                            // llamada por primera vez.
                            buscarCasillas(bt.obtenerPosicion(),true);
                            // Incrementamos en uno los clicks del mouse
                            clicks++;
                            // Establecemos en el estado la cantidad de clics
                            estado.setText("Clicks: "+clicks);
                            // Revisamos si se ha ganado el juego:
                            revisarBombas();
                        }
                    }
                    break;
            }
        } else if(obj == comando) { // Si el objeto que lo gener es el boton comando
            switch(estadoJuego) { // Revisamos en que estado se encuentra el juego
                case BuscaMinasApplet.ESTADO_JUEGO: // Si estamos en medio de una partida
                    reset(); // Reinicia el juego
                    break;
                case BuscaMinasApplet.ESTADO_ESPERA: // Si estamos al final de una partida ganada
                    ap.agregarPuntaje(new Puntaje(estado.getText(),clicks)); // Agregamos un nuevo puntaje
                    ap.mostrarPuntajes(); // Mostramos los puntajes
                    reset(); // Reiniciamos el juego
                    break;
                case BuscaMinasApplet.ESTADO_ESPERA2: // Si estamos al final de una partida perdida
                    reset(); // Reiniciamos el juego
                    break;
            }
        }
    }
    
    /**
     * Devuelve la informacin del applet.
     * @return Cadena de informacin del applet.
     */    
    public String getAppletInfo() {
        String msg = "BuscaMinasApplet Version 0.1.0\n"+
                     "Autor: Mario Gmez\n"+
                     "Licencia:\n"+
                     "Este Sofware esta protegido bajo la " +
                     "licencia GNU-GPL v2.0 o posterior" +
                     " que puede obtenerse en:\n"+
                     "http://www.fsf.org/.\n" +
                     "Se incluye una copia de la misma" +
                     " en el archivo LICENSE que se incluye" +
                     " con las fuentes";
        return msg;
    }
}
