/*
 * BuscaMinas.java
 *
 * Created on 26 de agosto de 2004, 09:59 PM
 */

package com.mxgxw.games;

/**
 * Esta clase define la lgica bsica del juego buscaminas. Puede ser utilizada
 * por cualquier programa para crear derivados del juego de buscaminas.
 * @author Mario Gmez
 * @version 0.1.0
 * @license Cdigo fuente protegido bajo la GNU-GPL o posterior que puede ser obtenida de http://www.fsf.org/. Se incluye una copia con las fuentes en el archivo LICENSE.
 */
public class BuscaMinas {
    
    /**
     * Constante que se utiliza para identificar una mina dentro del arreglo.
     */
    public static final int MINA = 0;
    /**
     * Constante que se utiliza para identificar un espacio vaco dentro del arreglo.
     */
    public static final int VACIO = 1;
    
    int[][] matrizJuego;
    int ancho,alto;
    /** Creates a new instance of BuscaMinas */
    public BuscaMinas() {
        int i,j;
        matrizJuego = new int[10][10];
        // Llena la matriz del juego con espacios vacos
        for(i=0;i<10;i++){
            for(j=0;j<10;j++){
                matrizJuego[j][i] = BuscaMinas.VACIO;
            }
        }
        ancho = alto = 10;
    }
    
    /**
     * Esta funcion se encarga de establecer todos los valores de la matriz de juego
     * a VACIO
     */
    public void reset() {
        int i,j;
        // Llena la matriz del juego con espacios vacos
        for(i=0;i<10;i++){
            for(j=0;j<10;j++){
                matrizJuego[j][i] = BuscaMinas.VACIO;
            }
        }
    }
    
    /**
     * Evalua si el rango especificado por val esta entre 0 y el limite definido
     * por lim
     * @return true si esta dentro del rango.
     * @param val Valor a evaluar.
     * @param lim Limite superior a evaluar.
     */
    public boolean revisarRango(int val,int lim) {
        boolean rangoCorrecto = false;
        // Revisa si el rango cumple la condicin
        if((val>=0)&&(val<lim)) {
            rangoCorrecto = true;
        }
        return rangoCorrecto;
    }
    
    /**
     * Cuenta la cantidad de minas alrededor de la posicin especificada por pos.
     * @return Numero de minas alrededor de la posicion especificada.
     * @param pos Posicin a evaluar para buscar minas.
     */
    public int radar(java.awt.Point pos) {
        int minas = 0,i,j;
        int x,y;
        // Revisa que la posicin solicitada cumpla con el rango
        if(revisarRango(pos.x,ancho)&&revisarRango(pos.y,alto)) {
            // Genera las posiciones para los nueve cuadros incluyendo
            // el cuadro que se revisa
            for(i=-1;i<2;i++){
                for(j=-1;j<2;j++){
                    x = pos.x+j;
                    y = pos.y+i;
                    // revisamos si la nueva posicion es vlida
                    if(revisarRango(x,ancho)&&revisarRango(y,alto)&&!(j==0&&i==0)) {
                        // Si existe una mina en una posicin aledaa
                        if(matrizJuego[y][x]==BuscaMinas.MINA)
                            minas++; // Aumentamos el contador
                    }
                }
            }
        }
        return minas;
    }
    
    /**
     * Revisa si existe una mina en la posicin indicada.
     * @return true si existe una mina.
     * @param pos Posicin a evaluar en busqueda de la mina.
     */
    public boolean hayMina(java.awt.Point pos) {
        boolean mina = false;
        // Revisa el rango
        if(revisarRango(pos.x,ancho)&&revisarRango(pos.y,alto)) {
            switch(matrizJuego[pos.y][pos.x]) {
                // si hay una mina regresa verdadero:
                case BuscaMinas.MINA: mina = true; break;
            }
        }
        return mina;
    }
    
    /**
     * Gener posiciones aleatorias para colocar las minas dentro de la matriz.
     */
    public void llenarCampo() {
        int i,x,y;
        for(i=0;i<10;i++) {
            do {
                // Generamos otra posicin aleatoria
                x = (int)Math.floor(Math.random()*9);
                y = (int)Math.floor(Math.random()*9);
            } while(hayMina(new java.awt.Point(x,y))); // Mientras haya una mina
            // en la posicin seleccionada
            // Una ves encontrada una posicin libre
            // Colocamos la mina
            matrizJuego[y][x] = BuscaMinas.MINA;
        }
    }
    
}
