
package com.mxgxw.graphics;

/**
 * La clase BotonSimple, representa un boton sencillo que puede poseer un texto o una imagen,
 * se pueden modificar tanto el color el texto o la imagen del mismo.
 * @author Mario Gmez
 * @version 0.1.0
 * @license Cdigo fuente protegido bajo la GNU-GPL o posterior que puede ser obtenida de http://www.fsf.org/. Se incluye una copia con las fuentes en el archivo LICENSE.
 */
public class BotonSimple extends java.awt.Button implements java.awt.image.ImageObserver {
    
    // Constantes de estado del boton
    /**
     * Constante que define el estado del boton elevado.
     */
    public static final int ELEVADO = 0;
    /**
     * Constante que define el estado de boton presionado
     */
    public static final int PRESIONADO = 1;
    
    // Constantes de configuracion del fondo
    /**
     * Constante que define un color como fondo.
     */
    public static final int VALOR_COLOR = 0;
    /**
     * Constante que define una imagen como fondo
     */
    public static final int VALOR_IMAGEN = 1;
    /**
     * Constante que define el fondo como un texto
     */
    public static final int VALOR_TEXTO = 2;
    
    // variables que llevan el registro del estado
    private int estado;
    
    private int valorFondo;
    
    // Colores del componente
    private java.awt.Color colorBrillo,colorSombra,colorFondo,colorBrilloFondo;
    private java.awt.Color colorTexto;
    
    // Imagen del componente
    private java.awt.Image imagenFondo;
    
    // Texto del componente
    private String msg;
    
    // Constructor por defecto
    /**
     * Constructor por defecto, se crea un boton con fondo estilo color y se inicializan los colores por defecto.
     */
    public BotonSimple() {
        llenarValoresPorDefecto();
        imagenFondo = new java.awt.image.BufferedImage(10,10,java.awt.image.BufferedImage.TYPE_INT_ARGB);
    }
    
    // Constructor con imagen
    /**
     * Define la imagen a ser mostrada cuando el boton tiene como fondo una imagen.
     * @param img Imagen proporcionada para el fondo del boton.
     */
    public BotonSimple(java.awt.Image img) {
        llenarValoresPorDefecto();
        imagenFondo = img;
    }
    
    // Establece los valores por defecto
    private void llenarValoresPorDefecto() {
        estado = BotonSimple.ELEVADO;
        valorFondo = BotonSimple.VALOR_COLOR;
        colorBrillo = new java.awt.Color(0xCC,0xCC,0xCC);
        colorSombra = new java.awt.Color(0x33,0x33,0x33);
        colorFondo = new java.awt.Color(0x99,0x99,0x99);
        colorBrilloFondo = new java.awt.Color(0xE1,0xE1,0xE1);
        colorTexto = new java.awt.Color(0xCC,0x00,0x00);
        msg = "\0";
    }
    
    
    /**
     * Cambia el estado del boton.
     * @param est Cambia el estado del boton, que puede tomar los valores de:
     * BotonSimple.ELEVADO: Muestra el boton elevado
     * BotonSimple.PRESIONADO: Muestra el boton presionado
     */
    public void ponerEstado(int est) {
        estado = est;
        this.repaint();
    }
    
    // Obtiene el estado del boton
    /**
     * Devuelve el estado actual del boton
     * @return Devuelve un int correspondiente al estado actual del boton.
     */
    public int obtenerEstado() {
        return estado;
    }
    
    // Establece el tipo de fondo
    /**
     * Cambia el tipo del fondo
     * @param tipo Define el tipo del fondo para el boton presionado, puede tomar los valores:
     * BotonSimple.VALOR_COLOR: Muestra el color de fondo
     * BotonSimple.VALOR_IMAGEN: Muestra la imagen de fondo
     * BotonSimple.VALOR_TEXTO: Muestra el texto de fondo
     * El color de fondo del botn siempre se mostrara, en imagen y texto se sobrepondra al color de fondo.
     */
    public void cambiarTipoFondo(int tipo) {
        valorFondo = tipo;
        repaint();
    }
    
    // Obtiene el tipo de fondo
    /**
     * Devuelve el tipo de fondo actual
     * @return valor del fondo actual
     */
    public int obtenerTipoFondo() {
        return valorFondo;
    }
    
    // Funciones para establecer los colores:
    /**
     * Establece el color del borde iluminado del boton.
     * @param c Color que se utilizara para dibujar la parte iluminada del boton.
     */    
    public void ponerColorBrillo(java.awt.Color c) {
        colorBrillo = c;
    }
    /**
     * Establece el color del borde oscuro del boton.
     * @param c Color que se utilizara para dibujar la parte oscura del boton.
     */    
    public void ponerColorSombra(java.awt.Color c) {
        colorSombra = c;
    }
    /**
     * Establece el color de fondo del boton.
     * @param c Color que se utilizara en el fondo del botn.
     */    
    public void ponerColorFondo(java.awt.Color c) {
        colorFondo = c;
    }
    /**
     * Establece el color del fondo cuando este esta presionado.
     * @param c Color que se utilizara en el fondo del botn cuando este esta presionado.
     */    
    public void ponerColorBrilloFondo(java.awt.Color c) {
        colorBrilloFondo = c;
    }
    /**
     * Establece el color del texto del botn.
     * @param c Color que se utilizara para dibujar el texto del botn.
     */    
    public void ponerColorTexto(java.awt.Color c) {
        colorTexto = c;
    }
    
    // Establece la imagen del componente:
    /**
     * Establece la imagen que se dibujar en el botn.
     * @param img Imagen que se dibujar en el botn.
     */    
    public void ponerImagen(java.awt.Image img) {
        imagenFondo = img;
    }
    
    // Establece el texto del componente:
    /**
     * Establece el texto del botn.
     * Es importante mencinar que por el tamao del mismo, solo se muestra el primer caracter,
     * como el botn tiene como finalidad mostrar un numero entre 1 y 8, para evitar que al
     * introducir un numero mas grande el aspecto cambie, automticamente se muestra nicamente
     * el primer caracter de la cadena especificada.
     * @param txt Cadena de texto que se mostrar en el boton.
     */    
    public void ponerTexto(String txt) {
        msg = txt;
    }
    /**
     * Funcion "alias" de la funcion ponerTexto. Para mantener compatibilidad con las
     * clases estndar de Java.
     * @param txt Cadena que se colocar en el botn.
     */    
    public void setText(String txt) {
        ponerTexto(txt);
    }
    
    // Funciones para obtener los colores:
    /**
     * Devuelve el color utilizado en la parte iluminada del boton.
     * @return Color utilizado
     */    
    public java.awt.Color obtenerColorBrillo() {
        return colorBrillo;
    }
    /**
     * Devuelve el color utilizado en la parte oscura del botn
     * @return Color utilizado
     */    
    public java.awt.Color obtenerColorSombra() {
        return colorSombra;
    }
    /**
     * Devuelve el color utilizado en el fondo del botn.
     * @return Color utilizado.
     */    
    public java.awt.Color obtenerColorFondo() {
        return colorFondo;
    }
    /**
     * Devuelve el color utilizado en el fondo del botn cuando esta presionado.
     * @return Color utilizado
     */    
    public java.awt.Color obtenerColorBrilloFondo() {
        return colorBrilloFondo;
    }
    /**
     * Devuelve el color utilizado para dibujar el texto.
     * @return Color utilizado
     */    
    public java.awt.Color obtenerColorTexto() {
        return colorTexto;
    }
    
    // Funcion para obtener la imagen:
    /**
     * Devuelve la imagen que esta utilizando el botn.
     * @return Imagen utilizada
     */    
    public java.awt.Image obtenerImagen() {
        return imagenFondo;
    }
    
    // Obtiene el texto del componente:
    /**
     * Devuelve la cadena de texto mostrada en el boton.
     * @return Devuelve la cadena de texto utilizada en el botn.
     */    
    public String obtenerTexto() {
        return msg;
    }
    /**
     * Funcin "alias" de obtenerTexto()
     * @return Devuelve la cadena de texto utilizada en el boton.
     */    
    public String getText() {
        return obtenerTexto();
    }
    
    // Informa del tamao del componente
    /**
     * Esta funcin define el tamao por defecto que tendr el componente al dibujarse
     * en la pantalla.
     * @return Tamao por defecto del componente.
     */    
    public java.awt.Dimension getPreferredSize() {
        return new java.awt.Dimension(20,20);
    }
    
    // Dibuja el nuevo componente
    /**
     * Esta funcin dibuja el componente.
     * @param g Entorno grfico en el que se dibujara el componente.
     */    
    public void paint(java.awt.Graphics g) {
        // Uso de Graphics2D para graficos avanzados
        java.awt.Graphics2D g2 = (java.awt.Graphics2D)g;
        
        // Activa el Antialiasing para mejorar la calidad de los grficos
        //g2.setRenderingHint(java.awt.RenderingHints.KEY_ANTIALIASING, java.awt.RenderingHints.VALUE_ANTIALIAS_ON);
        
        // Guarda el ancho y alto del Componente
        double width = this.getSize().getWidth();
        double height = this.getSize().getHeight();
        
        // Seleccion del color de fondo del cuadro:
        switch(estado) {
            case BotonSimple.ELEVADO:
                g2.setColor(colorFondo); break;
            case BotonSimple.PRESIONADO:
                g2.setColor(colorBrilloFondo); break;
        }
        // Nuevo Rectangulo relleno:
        g2.fill(new java.awt.geom.Rectangle2D.Double(0,0,width,height));
        
        // Al presionar selecciona entre mostrar el texto, la imagen o el cuadro
        // vaco.
        switch(valorFondo) {
            case BotonSimple.VALOR_IMAGEN:
                // Calcula la pocision de la imagen
                // posx = ancho/2 + anchoImagen/2
                // posy = alto/2 + altoImagen/2
                double imgX = width/2 - imagenFondo.getWidth(this)/2;
                double imgY = height/2 - imagenFondo.getHeight(this)/2;
                // Dibuja la imagen:
                g2.drawImage(imagenFondo,(int)imgX,(int)imgY,this);
                break;
            case BotonSimple.VALOR_TEXTO:
                // Selecciona el color del texto:
                g2.setColor(colorTexto);
                // Establece la fuente a usar y el tamao:
                g2.setFont(new java.awt.Font("SansSerif",java.awt.Font.BOLD,16));
                // Obtenemos las metricas de la fuente para
                // calcular la pocision:
                java.awt.FontMetrics fm = g2.getFontMetrics();
                
                // Obtenemos las caracteristicas de la fuente
                int msgWidth = fm.stringWidth(msg);
                int ascent = fm.getMaxAscent();
                int descent = fm.getMaxDescent();
                
                // Calculamos las pocisiones de la fuente:
                double msgX = width/2 - msgWidth/2;
                double msgY = height/2 - descent/2 + ascent/2;
                
                // Mostramos solo el primer caracter de la cadena
                // esto es porque si mostraramos mas se saldrian del
                // contenedor
                g2.drawString(msg.substring(0,1),(int)msgX,(int)msgY);
                break;
        }
        
        // Establece las propiedades para dibujar las lineas:
        g2.setStroke(new java.awt.BasicStroke(3));
        
        // Selecciona el color de las lineas inferiores:
        switch(estado) {
            case BotonSimple.ELEVADO:
                g2.setColor(colorSombra);
                // Dibuja las lineas inferiores
                g2.draw(new java.awt.geom.Line2D.Double(0,height,width,height));
                g2.draw(new java.awt.geom.Line2D.Double(width,0,width,height));
                break;
        }
        
        
        // Selecciona el color de las lineas superiores:
        switch(estado) {
            case BotonSimple.ELEVADO:
                g2.setStroke(new java.awt.BasicStroke(3));
                g2.setColor(colorBrillo);
                break;
            case BotonSimple.PRESIONADO:
                g2.setStroke(new java.awt.BasicStroke(2));
                g2.setColor(colorSombra); break;
        }
        // Dibuja las lineas superiores:
        g2.draw(new java.awt.geom.Line2D.Double(0,0,width,0));
        g2.draw(new java.awt.geom.Line2D.Double(0,0,0,height));
    }
    
}
